@@ -2,7 +2,7 @@ __author__ = "Vini Salazar"
 __license__ = "MIT"
 __maintainer__ = "Vini Salazar"
 __url__ = "https://github.com/vinisalazar/bioprov"
-__version__ = "0.1.15"
+__version__ = "0.1.17"
 
 """
 
@@ -25,6 +25,7 @@ This class also contains functions to read and write objects in JSON and tab-del
 import datetime
 import json
 import pandas as pd
+import tempfile
 from bioprov import config
 from bioprov.utils import Warnings, serializer, serializer_filter, dict_to_sha1
 from bioprov.src.files import File, SeqFile, deserialize_files_dict
@@ -103,7 +104,7 @@ class Program:
         :param runs: See input to add_runs function.
         :return: Adds runs to Sample
         """
-        add_runs(self, runs)
+        _add_runs(self, runs)
 
     def generate_cmd(self):
         """
@@ -305,11 +306,11 @@ class Run:
 
     @property
     def status(self):
-        return self._finished_to_status(self.finished)
+        self._status = self._finished_to_status(self.finished)
+        return self._status
 
     @status.setter
-    def status(self, _):
-        value = self._finished_to_status(self.finished)
+    def status(self, value):
         self._status = value
 
     @staticmethod
@@ -386,7 +387,7 @@ class Run:
         duration = str(datetime.timedelta(seconds=duration))
         self.duration = duration
         self.finished = True
-        self.status = self._finished_to_status(self.finished)
+        self._status = self._finished_to_status(self.finished)
 
         # These are useful for quick debugging.
         if _print_stdout:
@@ -684,7 +685,7 @@ def generate_param_str(params):
     return param_str
 
 
-def add_programs(object_, programs):
+def _add_programs(object_, programs):
     """
     Adds program(s) to object. Must be an instance or iterable of bioprov.Program.
     :param object_: A bioprov.Sample or Project instance.
@@ -716,7 +717,7 @@ def add_programs(object_, programs):
         object_.programs[program.name] = program
 
 
-def add_runs(object_, runs):
+def _add_runs(object_, runs):
     """
     Adds run(s) to object. Must be an instance or iterable of bioprov.Run.
     :param object_: A bioprov.Sample or Project instance.
@@ -784,7 +785,7 @@ class Sample:
         else:
             attributes = dict()
         self.attributes = attributes
-        self._programs = None
+        self.programs = OrderedDict()
 
         # This is an attribute used by the src.prov module
         self.namespace_preffix = f"samples:{self.name}"
@@ -810,7 +811,7 @@ class Sample:
                          and value is a bp.Program instance.
         :return: Updates self by adding the programs to object.
         """
-        add_programs(self, programs)
+        _add_programs(self, programs)
 
     def add_files(self, files):
         """
@@ -818,16 +819,14 @@ class Sample:
         :param files: See input to add_files function.
         :return: Adds files to Sample
         """
-        add_files(self, files)
+        _add_files(self, files)
 
     def serializer(self):
         """
         Custom serializer for Sample class. Serializes runs, programs, and files attributes.
         :return:
         """
-        keys = [
-            "files_namespace_preffix",
-        ]
+        keys = ["files_namespace_preffix"]
         return serializer_filter(self, keys)
 
     def run_programs(self, _print=True):
@@ -835,30 +834,16 @@ class Sample:
         Runs self._programs in order.
         :return:
         """
-        if len(self.programs) >= 1:
-            for _, p in self.programs.items():
-                self._run_program(p, _print=_print)
-        else:
-            print(f"No programs to run for Sample '{self.name}'")
+        _run_programs(self, _print)
 
     def _run_program(self, program, _print=True):
         """
-        Run a Program or PresetProgram on Sample.
-        :param program: An instance of bioprov.Program or PresetProgram
+        Runs program for self.
+        :param program: bioprov.Program or bioprov.PresetProgram.
         :param _print: Whether to print output of Program.
-        :return: Runs the program for Sample.
+        :return: Runs Program and updates self.
         """
-        program.run(sample=self, _print=_print)
-        self.add_programs(program)
-
-        if program not in self.programs:
-            self.programs[program.name] = program
-
-    @property
-    def programs(self):
-        if self._programs is None:
-            self._programs = dict()
-        return self._programs
+        _run_program(self, program, _print)
 
     def to_json(self, _path=None, _print=True):
         """
@@ -876,7 +861,17 @@ class Sample:
         :return: pd.Series
         """
         series = {}
-        for k, v in self.__dict__.items():
+
+        # Can't apply serializer_filter here.
+        keys = ["files_namespace_preffix", "namespace_preffix", "_programs"]
+        modified_dict = self.__dict__.copy()
+        for key in keys:
+            try:
+                del modified_dict[key]
+            except KeyError:
+                pass
+
+        for k, v in modified_dict.items():
             if v is None or not v:
                 continue
             if isinstance(v, dict) and len(v) > 0:
@@ -888,6 +883,36 @@ class Sample:
         return pd.Series(series)
 
 
+def _run_program(_object, program, _print=True):
+    """
+    Runs program for _object.
+
+    :param _object: bioprov.Project or bioprov.Sample
+    :param program: bioprov.Program or bioprov.PresetProgram.
+    :param _print: Whether to print output of Program.
+    :return: Runs Program and updates _object.
+    """
+    if program not in _object.programs.keys():
+        _object.add_programs(program)
+    program.run(sample=_object, _print=_print)
+
+
+def _run_programs(_object, _print=True):
+    """
+    Runs programs in order.
+
+    :param _object: bioprov.Project or bioprov.Sample
+    :param _print: Whether to print output of Program.
+    :return:
+    """
+    if len(_object.programs) >= 1:
+        for _, p in _object.programs.items():
+            # noinspection PyProtectedMember
+            _object._run_program(p, _print=_print)
+    else:
+        print(f"No programs to run for {_object}")
+
+
 class Project:
     """
     Class which holds a dictionary of Sample instances, where each key is the sample name.
@@ -905,7 +930,9 @@ class Project:
         if tag is None:
             tag = generate_slug(2)
         self.tag = tag.replace(" ", "_")
+        self._name = self.tag
         self.files = dict()
+        self.programs = OrderedDict()
         samples = self.is_iterator(
             samples
         )  # Checks if `samples` is a valid constructor.
@@ -919,6 +946,8 @@ class Project:
         # PROV attributes
         self._entity = None
         self._bundle = None
+        self.namespace_preffix = f"project:{self}"
+        self.files_namespace_preffix = None
 
         # Hash and db attributes
         self._sha1 = dict_to_sha1(self.serializer())
@@ -938,12 +967,9 @@ class Project:
             value = self._samples[item]
             return value
         except KeyError:
-            keys = self.keys()
-            print(
-                f"Sample {item} not in Project.\n",
-                "Check the following keys:\n",
-                "\n".join(keys),
-            )
+            print(f"Sample {item} not in Project.\n")
+            print("Check the following keys:")
+            print(" ", "\n  ".join(self.keys))
 
     def __setitem__(self, key, value):
         self._samples[key] = value
@@ -960,6 +986,11 @@ class Project:
     def items(self):
         return self._samples.items()
 
+    @property
+    def name(self):
+        self._name = self.tag
+        return self._name
+
     @property
     def sha1(self):
         keys = ("_sha1",)
@@ -1045,13 +1076,41 @@ class Project:
 
     def add_files(self, files):
         """
-        Project method to add files.
-        :param files: See input to add_files function.
-        :return: Adds files to Project
+        Adds Files to self.files. See documentation to bioprov.src.main.add_files().
+
+        :param files: Dict, list, or File instance.
+        :return: Updates self.files
         """
-        add_files(self, files)
+        _add_files(self, files)
         self.auto_update_db()
 
+    def add_programs(self, programs):
+        """
+        Add programs to self. See documentation to bioprov.src.main.Progarm
+        :param programs: Dict, list, or Program instance.
+        :return: Updates self.programs
+        """
+        _add_programs(self, programs)
+        self.auto_update_db()
+
+    def run_programs(self, _print=True):
+        """
+        Runs all programs in self.programs in order.
+
+        :return:
+        """
+        _run_programs(self, _print)
+
+    def _run_program(self, program=None, _print=True):
+        """
+        Runs program for self.
+
+        :param program: bioprov.Program or bioprov.PresetProgram.
+        :param _print: Whether to print output of Program.
+        :return: Runs Program and updates self.
+        """
+        _run_program(self, program, _print)
+
     @staticmethod
     def is_sample_and_name(sample):
         """
@@ -1167,7 +1226,7 @@ class Project:
         df.to_csv(path_, sep=sep, **kwargs)
 
 
-def add_files(object_, files):
+def _add_files(object_, files):
     """
     Adds file(s) to object. Must be a dict or an instance or iterable of bioprov.File.
     :param object_: A Sample or Project instance.
@@ -1267,32 +1326,49 @@ def from_json(json_file, kind="Project", replace_path=None, replace_home=False):
             HOME = str(Path.home())
 
         samples = dict()
-        for k, v in d["_samples"].items():
-            samples[k] = dict_to_sample(v)
+
+        try:
+            for k, v in d["_samples"].items():
+                samples[k] = dict_to_sample(v)
+        except KeyError:
+            pass
 
         # Create Project
         project = Project(samples=samples, tag=d["tag"])
 
-        # Deserializing and adding project files
-        deserialized_files = deserialize_files_dict(d["files"])
-        deque((project.add_files(file_) for file_ in deserialized_files.values()))
+        # Deserializing and adding project files and programs
+        try:
+            deserialized_files = deserialize_files_dict(d["files"])
+            deque((project.add_files(file_) for file_ in deserialized_files.values()))
+            deserialized_programs = deserialize_programs_dict(d["programs"], project)
+            deque(
+                (
+                    project.add_programs(program_)
+                    for program_ in deserialized_programs.values()
+                )
+            )
+        except KeyError:
+            pass
 
-        for user, env in d["users"].items():
-            for env_hash, env_dict in env.items():
-                try:
-                    project.users[user][env_hash] = EnvProv()
-                except KeyError:
-                    project.users[user] = dict()
-                    project.users[user][env_hash] = EnvProv()
-                for env_attr_, attr_value_ in env_dict.items():
-                    if env_attr_ == "env_namespace":
-                        attr_value_ = Namespace(
-                            "envs", str(project.users[user][env_hash])
-                        )
-                    if replace_home:
-                        if env_attr_ == "env_dict":
-                            other_HOME_variables.append(attr_value_["HOME"])
-                    setattr(project.users[user][env_hash], env_attr_, attr_value_)
+        try:
+            for user, env in d["users"].items():
+                for env_hash, env_dict in env.items():
+                    try:
+                        project.users[user][env_hash] = EnvProv()
+                    except KeyError:
+                        project.users[user] = dict()
+                        project.users[user][env_hash] = EnvProv()
+                    for env_attr_, attr_value_ in env_dict.items():
+                        if env_attr_ == "env_namespace":
+                            attr_value_ = Namespace(
+                                "envs", str(project.users[user][env_hash])
+                            )
+                        if replace_home:
+                            if env_attr_ == "env_dict":
+                                other_HOME_variables.append(attr_value_["HOME"])
+                        setattr(project.users[user][env_hash], env_attr_, attr_value_)
+        except KeyError:
+            pass
 
         if replace_home:
             project.replace_paths(other_HOME_variables, HOME, warnings=True)
@@ -1389,8 +1465,8 @@ def read_csv(df_path, sep=",", **kwargs):
     :return: A Project instance.
     """
     df = pd.read_csv(df_path, sep=sep)
-    sampleset = from_df(df, source_file=df_path, **kwargs)
-    return sampleset
+    project = from_df(df, source_file=df_path, **kwargs)
+    return project
 
 
 def json_to_dict(json_file):
@@ -1426,7 +1502,7 @@ def dict_to_sample(json_dict):
                 )
 
             # Create Program instances
-            elif attr == "_programs":
+            elif attr == "programs":
                 deserialized_programs = deserialize_programs_dict(value, sample_)
                 deque(
                     (
@@ -1446,7 +1522,7 @@ def write_json(dict_, _path, _print=True):
     :param dict_: JSON dictionary.
     :param _path: String with _path to JSON file.
     :param _print: Whether to print if the file was successfully created.
-    :return: Writes JSON file
+    :return: Writes JSON file.
     """
     with open(_path, "w") as f:
         json.dump(dict_, f, indent=3)
@@ -1456,3 +1532,28 @@ def write_json(dict_, _path, _print=True):
             print(f"Created JSON file at {_path}.")
         else:
             print(f"Could not create JSON file for {_path}.")
+
+
+def load_project(tag):
+    """
+    Loads Project from the BioProvDatabase set in the config.
+
+    :param tag: Tag of the Project to be loaded.
+    :return: Instance of Project.
+    """
+    assert (
+        len(config.db) > 0
+    ), f"Project not found. Database at '{config.db_path}' is empty"
+
+    query = Query()
+    try:
+        result = config.db.search(query.tag == tag)[0]
+    except (IndexError, KeyError):
+        print(f"Project not found in database at {config.db_path}")
+        return
+
+    with tempfile.NamedTemporaryFile() as f:
+        f.write(bytes(json.dumps(result), "utf-8"))
+        project = from_json(f.name)
+
+    return project
